"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const effects_1 = require("redux-saga/effects");
exports.concoctBoilerplate = (sideEffects, actionTypes, defaultState) => {
    const keys = Object.keys(sideEffects);
    const actions = keys.reduce((acc, k) => {
        // tslint:disable-next-line: one-variable-per-declaration
        const reducer = (state = defaultState, action) => {
            switch (action.type) {
                case actionTypes[k][0]:
                    return {
                        ...state,
                        loading: true
                    };
                case actionTypes[k][1]:
                    return {
                        ...state,
                        loading: false,
                        result: action.payload !== undefined ? action.payload.result : undefined,
                        error: undefined
                    };
                case actionTypes[k][2]:
                    return {
                        ...state,
                        loading: false,
                        result: undefined,
                        error: action.payload !== undefined ? action.payload.error : undefined
                    };
                default:
                    return state;
            }
        };
        function* saga(action) {
            // tslint:disable-next-line: no-try
            try {
                const result = yield effects_1.call(sideEffects[k], ...action.payload.params);
                // tslint:disable-next-line: no-expression-statement
                yield effects_1.put({
                    type: actionTypes[k][1],
                    payload: {
                        result
                    }
                });
            }
            catch (error) {
                // tslint:disable-next-line: no-expression-statement
                yield effects_1.put({
                    type: actionTypes[k][2],
                    payload: {
                        error
                    }
                });
            }
        }
        return {
            ...acc,
            [k]: [
                // PERFORM SIDE EFFECT (e.g. GET)
                (...params) => ({
                    type: actionTypes[k][0],
                    payload: {
                        params
                    }
                }),
                // PERFORM SIDE EFFECT SUCCESS (e.g. GET SUCCESS)
                (result) => ({
                    type: actionTypes[k][1],
                    payload: {
                        result
                    }
                }),
                // PERFORM SIDE EFFECT FAILURE (e.g. GET FAILURE)
                (error) => ({
                    type: actionTypes[k][2],
                    payload: {
                        error
                    },
                    error: true
                }),
                reducer,
                saga
            ]
        };
        // TODO get rid of this cast
    }, {});
    const rootReducer = keys.reduce((acc, k) => ({
        ...acc,
        [actionTypes[k][3]]: actions[k][3]
    }), {});
    function* rootSaga() {
        // tslint:disable-next-line: no-expression-statement
        yield effects_1.all(keys.map(k => effects_1.takeLatest(actionTypes[k][0], actions[k][4])));
    }
    return {
        actions,
        rootReducer,
        rootSaga
    };
};
